import re
from datetime import tzinfo
from typing import Any
from typing import Callable
from typing import Container
from typing import Iterator
from typing import Literal
from typing import Mapping
from typing import Sequence
from typing import TypeVar
from typing import overload

T = TypeVar("T")
Decorator = Callable[[T], T]
TypeConverter = Callable[[str], T]

__version__: str
__all__ = [
	"parse",
	"search",
	"findall",
	"with_pattern",
]


class TooManyFields(ValueError): ...
class RepeatedNameError(ValueError): ...


def extract_format(format: str, extra_types: Container[str]) -> dict[str, Any]: ...
def with_pattern(pattern: str, regex_group_count: int = ...) -> Decorator[Callable[..., Any]]: ...
def compile(format: str, extra_types: Container[str] = ..., case_sensitive: bool = ...) -> Parser: ...


# To keep the number of possible overload variations within manageable limits the
# evaluate_result arguments of these functions and their Parse.* equivalents are keyword
# ONLY.

@overload
def parse(
	format: str,
	string: str,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[True] = True,
) -> Result|None: ...

@overload
def parse(
	format: str,
	string: str,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[False],
) -> Match|None: ...

@overload
def search(
	format: str,
	string: str,
	pos: int = ...,
	endpos: int = ...,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[True] = True,
) -> Result|None: ...

@overload
def search(
	format: str,
	string: str,
	pos: int = ...,
	endpos: int = ...,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[False],
) -> Match|None: ...

@overload
def findall(
	format: str,
	pos: int = ...,
	endpos: int = ...,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[True] = True,
) -> Iterator[Result]: ...

@overload
def findall(
	format: str,
	pos: int = ...,
	endpos: int = ...,
	extra_types: Container[str] = ...,
	case_sensitive: bool = ...,
	*, evaluate_result: Literal[False],
) -> Iterator[Match]: ...

# def percentage(string: str, match: re.Match[str]) -> float: ...
# def date_convert(
# 	string: str,
# 	match: re.Match[str],
# 	ymd: int = ...,
# 	mdy: int = ...,
# 	dmy: int = ...,
# 	d_m_y: int = ...,
# 	hms: int = ...,
# 	am: int = ...,
# 	tz: int = ...,
# 	mm: int = ...,
# 	dd: int = ...,
# ) -> datetime: ...


# class int_convert:

# 	base: int|None

# 	def __init__(self, base: int = ...): ...
# 	def __call__(self, string: str, match: re.Match[str]) -> int: ...


# class convert_first(Generic[T]):

# 	converter: Callable[[str], T]

# 	def __init__(self, converter: Callable[[str], T]): ...
# 	def __call__(self, string: str, match: re.Match[str]) -> T: ...


class FixedTzOffset(tzinfo):

	def __init__(self, offset: int, name: str): ...
	def __eq__(self, other: Any) -> bool: ...


class Parser:

	@property
	def named_fields(self) -> list[str]: ...

	@property
	def fixed_fields(self) -> list[int]: ...

	def __init__(self, format: str, extra_types: Container[str] = ..., case_sensitive: bool = ...): ...

	def evaluate_result(self, m: re.Match[str]) -> Result: ...

	@overload
	def parse(
		self,
		string: str,
		*, evaluate_result: Literal[True] = True,
	) -> Result|None: ...

	@overload
	def parse(
		self,
		string: str,
		*, evaluate_result: Literal[False],
	) -> Match|None: ...

	@overload
	def search(
		self,
		string: str,
		pos: int = ...,
		endpos: int = ...,
		*, evaluate_result: Literal[True] = True,
	) -> Result|None: ...

	@overload
	def search(
		self,
		string: str,
		pos: int = ...,
		endpos: int = ...,
		*, evaluate_result: Literal[False],
	) -> Match|None: ...

	@overload
	def findall(
		self,
		pos: int = ...,
		endpos: int = ...,
		extra_types: Container[str] = ...,
		*, evaluate_result: Literal[True] = True,
	) -> Iterator[Result]: ...

	@overload
	def findall(
		self,
		pos: int = ...,
		endpos: int = ...,
		extra_types: Container[str] = ...,
		*, evaluate_result: Literal[False],
	) -> Iterator[Match]: ...


class Result:

	fixed: Sequence[Any]
	named: Mapping[str, Any]
	spans: Mapping[str|int, tuple[int, int]]

	def __init__(
		self,
		fixed: Sequence[Any],
		names: Mapping[str, Any],
		spans: Mapping[str|int, tuple[int, int]],
	): ...

	def __getattr__(self, item: str|int) -> Any: ...
	def __contains__(self, name: str) -> bool: ...


class Match:

	def __init__(self, parser: Parser, match: re.Match[str]): ...
	def evaluate_result(self) -> Result: ...
